macro_rules! impl_from_variants {
    ($enum_type:ident, $($variant:ident($inner_type:ty)),*) => {
        $(
            impl From<$inner_type> for $enum_type {
                fn from(inner: $inner_type) -> Self {
                    $enum_type::$variant(inner)
                }
            }
        )*
    };
}

macro_rules! impl_wrapper_err_for {
    ($name:ident($inner:ty), $err:literal) => {
        impl_wrapper_err!($name, $err);
        impl_from_ty!($name, $inner);
    };
}

macro_rules! impl_from_ty {
    ($name:ty, $from:ty) => {
        impl From<$from> for $name {
            fn from(value: $from) -> Self {
                Self(value)
            }
        }
    };
}

macro_rules! impl_wrapper_err {
    ($type:ty, $err:literal) => {
        impl Display for $type {
            fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
                write!(f, "{}: {}", self.0, $err)
            }
        }

        impl Error for $type {}
    };
}

pub(crate) use impl_from_ty;
pub(crate) use impl_from_variants;
pub(crate) use impl_wrapper_err;
pub(crate) use impl_wrapper_err_for;
